import Actor
import Box2D
import pygame
import DebugDraw
import IcarusActor
import WallActor
import WindActor
import ThermalActor
import random
import math

class FeatherActor(Actor.Actor):
    
    featherBody = None
    featherAnchorBody = None
    joint = None
    featherimage = None
    untouchableTimer = 30 # while non-zero, feather can't be picked up
    
    def __init__(self, world, debugDraw, position, featherimage):    
        
        self.featherimage = featherimage
        
        # set up a body/shape for the feather         
        featherAnchorBodyDef = Box2D.b2BodyDef()
        featherAnchorBodyDef.position = position
        featherAnchorBodyDef.fixedRotation = True
        featherAnchorBodyDef.linearDamping = 2
        self.featherAnchorBody = world.CreateBody(featherAnchorBodyDef)
        self.featherAnchorBody.SetLinearVelocity((random.uniform(-10, 10), random.uniform(-10, 10)))

        featherAnchorShapeDef = Box2D.b2CircleDef()
        featherAnchorShapeDef.radius = 1.5
        featherAnchorShapeDef.density = 0.3
        featherAnchorShapeDef.friction = 1
        featherAnchorShapeDef.restitution = 1
        featherAnchorShapeDef.filter.categoryBits = 0 # Don't want the anchor to collide with anything 
        self.featherAnchorBody.CreateShape(featherAnchorShapeDef)
        self.featherAnchorBody.SetMassFromShapes()


        featherBodyDef = Box2D.b2BodyDef()
        #featherBodyDef.position = (position[0], position[1] - 1.5)
        featherBodyDef.position = position + (0, -1.5)
        #featherBodyDef.fixedRotation = True
        featherBodyDef.linearDamping = 1
        self.featherBody = world.CreateBody(featherBodyDef)

        featherShapeDef = Box2D.b2CircleDef()
        featherShapeDef.radius = 1
        featherShapeDef.density = 0.1
        featherShapeDef.friction = 1
        featherShapeDef.restitution = 1
        featherShapeDef.filter.categoryBits = Actor.CollisionGroup.feather
        featherShapeDef.filter.maskBits = Actor.CollisionGroup.icarus_wings | Actor.CollisionGroup.wall | Actor.CollisionGroup.wind | Actor.CollisionGroup.thermal
        self.featherBody.CreateShape(featherShapeDef)
        self.featherBody.SetMassFromShapes()

        
        jointDef = Box2D.b2RevoluteJointDef()
        jointDef.Initialize(self.featherBody, self.featherAnchorBody, position)        
        jointDef.lowerAngle    = -0.25 * Box2D.b2_pi  # -45 degrees
        jointDef.upperAngle    = 0.25 * Box2D.b2_pi  #  45 degrees
        jointDef.enableLimit   = True
        jointDef.maxMotorTorque= 10.0
        jointDef.motorSpeed    = 2.0
        jointDef.enableMotor   = True
        self.joint = world.CreateJoint(jointDef).getAsType()

        # Call the base constructor, with our body
        super(FeatherActor, self).__init__(self.featherBody, debugDraw, world)
    
    def CanBeCollected(self):
        return self.untouchableTimer == 0
    
    def HandleContact(self, contact):
        if self.isAlive == False:
            return

        if contact.shape1.GetBody().userData != self:
            print "Error: FeatherActor.HandleContact called, but I am not shape1 owner"
            return

        actor2 = contact.shape2.GetBody().userData
        
        if isinstance(actor2, WindActor.WindActor):
            actor2.HandleTouchedByFeather(self)
        elif isinstance(actor2, ThermalActor.ThermalActor):
            actor2.HandleTouchedByFeather(self)

    def Update(self, keys):
        if self.untouchableTimer != 0:
            self.untouchableTimer -= 1
        
        # feather should not be affected by gravity, so apply an upward force to counteract its weight
        p = self.featherBody.GetWorldPoint((0.0, 0.0))
        weight = self.featherBody.GetMass() * self.world.GetGravity()
        self.featherBody.ApplyForce(-weight, p)
        # and apply one to the anchor, but only against half its weight, to make it more floaty
        p = self.featherAnchorBody.GetWorldPoint((0.0, 0.0))
        weight = self.featherAnchorBody.GetMass() * self.world.GetGravity() / 2
        self.featherAnchorBody.ApplyForce(-weight, p)
        
        if self.joint.GetJointAngle() > 0.15 * Box2D.b2_pi:
            self.joint.motorSpeed = -1
        elif self.joint.GetJointAngle() < -0.15 * Box2D.b2_pi:
            self.joint.motorSpeed = 1
        
        if self.body is not None:
            self.renderpos = (self.body.position.x, self.body.position.y) 
        
    def Render(self, doDebugDraw):    
        if self.body is None:
            return        
        a = self.featherBody.GetWorldPoint((-1, 0))   
        b = self.featherBody.GetWorldPoint((1, 0))        
        
        # rotate feather image by current angle
        diff = a - b
        angle = math.degrees(math.atan2(diff.y, diff.x))
        rotfeather = pygame.transform.rotate(self.featherimage, angle)
        self.debugDraw.DrawImage(
             rotfeather, 
             (self.body.position.x, self.body.position.y),
             self.featherimage.get_width() / 2,
             self.featherimage.get_height() /2)
        
        
        #self.debugDraw.DrawCircle(a, 1, (0.8, 0.8, 0.8))
        #self.debugDraw.DrawCircle(b, 1, (0.8, 0.8, 0.8))

#        for shape in self.featherBody.GetShapeList(True):
#            center = self.featherBody.GetWorldPoint(shape.GetLocalPosition())
#            radius = shape.GetRadius()
#            color = Box2D.b2Color(0.8, 0.1, 0.1)
#            self.debugDraw.DrawCircle(center, radius, color)
#        
#        for shape in self.featherAnchorBody.GetShapeList(True):
#            center = self.featherAnchorBody.GetWorldPoint(shape.GetLocalPosition())
#            radius = shape.GetRadius()
#            color = Box2D.b2Color(0.1, 0.8, 0.1)
#            self.debugDraw.DrawCircle(center, radius, color)

        super(FeatherActor, self).Render(doDebugDraw)
        
        
    def Kill(self):
        if self.isAlive:
            # Need to manually kill the anchor body, since the base class doesn't know about it
            self.world.DestroyBody(self.featherAnchorBody)
            # then call the base method
            super(FeatherActor, self).Kill()

        