'''
Created on 31 Aug 2009

@author: jamie
'''

import os
import random
import pygame
import Box2D
import DebugDraw
import Actor
import IcarusActor
import WallActor
import BirdActor
import TriggerActor
import WindActor
import ThermalActor
import BulletActor
import FeatherActor
import ParticleActor
import svgparser
import CreditsRenderer

from pygame.locals import *
from os import path
from copy import copy

class fwDestructionListener(Box2D.b2DestructionListener):
    """
    The destruction listener callback:
    "SayGoodbye" is called when a joint or shape is deleted.
    """
    def __init__(self):
        super(fwDestructionListener, self).__init__()

    def SayGoodbye(self, object):
        # Do nothing (yet)
        return

class fwBoundaryListener(Box2D.b2BoundaryListener):
    """
    The boundary listener callback:
    Violation is called when the specified body leaves the world AABB.
    """
    game = None
    
    def __init__(self, game):
        super(fwBoundaryListener, self).__init__()
        self.game = game

    def Violation(self, body):
        if isinstance(body.userData, BulletActor.BulletActor):
            return
        self.game.destroyList.append(body)

class fwContactTypes:
    """
    Acts as an enum, holding the types necessary for contacts:
    Added, persisted, and removed
    """
    contactUnknown = 0
    contactAdded = 1
    contactPersisted = 2
    contactRemoved = 3

class fwContactPoint:
    """
    Structure holding the necessary information for a contact point.
    All of the information is copied from the contact listener callbacks.
    """
    shape1 = None
    shape2 = None
    actor1 = None
    actor2 = None
    normal = None
    position = None
    velocity = None
    id  = None
    state = 0
    
    def GetSwapped(self):
        """
        Returns a copy of this object, but with the shapes and the normal inverted
        """
        swappedContact = fwContactPoint()
        swappedContact.shape1 = self.shape2
        swappedContact.shape2 = self.shape1
        swappedContact.actor1 = self.actor2
        swappedContact.actor2 = self.actor1
        swappedContact.normal = self.normal * -1
        swappedContact.position = self.position
        swappedContact.velocity = self.velocity
        swappedContact.id = self.id
        swappedContact.state = self.state
        return swappedContact
    
class fwContactListener(Box2D.b2ContactListener):
    """
    Handles all of the contact states passed in from Box2D.
    """
    
    game = None
    
    def __init__(self, game):
        super(fwContactListener, self).__init__()
        self.game = game

    def handleCall(self, state, point):
        if not self.game: return

        cp          = fwContactPoint()
        cp.shape1   = point.shape1
        cp.shape2   = point.shape2
        cp.actor1   = point.shape1.GetBody().userData
        cp.actor2   = point.shape2.GetBody().userData
        cp.position = point.position.copy()
        cp.normal   = point.normal.copy()
        cp.id       = point.id
        cp.state    = state
        self.game.contactPoints.append(cp)

    def Add(self, point):
        self.handleCall(fwContactTypes.contactAdded, point)

    def Persist(self, point):
        self.handleCall(fwContactTypes.contactPersisted, point)

    def Remove(self, point):
        self.handleCall(fwContactTypes.contactRemoved, point)

class LevelDefinition:
    """
    Structure holding information about a particular level
    """
    dataFileName = None
    title = None
    nextLevel = None
    startPoint = None
    initialCameraPoint = None

class eGameState:
    """
    Acts as an enum, holding current game state
    """
    stateTitle = 0
    stateLevelTransition = 1
    statePlayingLevel = 2
    stateYouWin = 3

class Game(object):
    # Box2D-related
    worldAABB          = None
    world              = None
    contactPoints      = []
    destroyList        = []
    
    # Box2D-callbacks
    destructionListener= None
    boundaryListener   = None
    contactListener    = None
    debugDraw          = None
        
    # Screen/rendering-related
    _viewZoom          = 10.0
    _viewCenter        = None
    _viewOffset        = None
    screenSize         = None
    rMouseDown         = False
    textLine           = 60
    fps                = 0
    
    fontFPS              = None # font used to render the FPS display
    fontDefault              = None # default font used to render text
    fontTransition              = None # font used to render the transition text

    # Update rate (Hz)
    updateRate = 60
    
    icarus = None
    actors = []
    
    levels = []
    currentGameState = eGameState.stateTitle
    currentLevel = None
    nextLevel = None
    creditsRenderer = None
    ignoreKeyTimer = 0
    
    # animation stuff
    artdir = path.abspath("art/")
    birdframes = []
    icarusframesRight = []
    icarusframesLeft = []
    feathers = []
    
    # sound stuff
    sounddir = path.abspath("sound")
    soundbank = {}
    specialsound = None

    def __init__(self):
        # Box2D Initialization
        self.worldAABB=Box2D.b2AABB()
        self.worldAABB.lowerBound = (-2000.0, -1000.0)
        self.worldAABB.upperBound = ( 2000.0, 2000.0)
        
        gravity = (0.0, -10.0)
        doSleep = True
        self.world = Box2D.b2World(self.worldAABB, gravity, doSleep)
        self.destructionListener = fwDestructionListener()
        self.boundaryListener = fwBoundaryListener(self)
        self.contactListener = fwContactListener(self)
        self.debugDraw = DebugDraw.fwDebugDraw()
        
        self.world.SetDestructionListener(self.destructionListener)
        self.world.SetBoundaryListener(self.boundaryListener)
        self.world.SetContactListener(self.contactListener)
    
        # Pygame Initialization        
        os.environ['SDL_VIDEO_CENTERED'] = '1'
        pygame.mixer.init()
        pygame.init()
        
        pygame.key.set_repeat()        
    
        caption= "Icarus"
        pygame.display.set_caption(caption)
    
        self.screen = pygame.display.set_mode( (1000,600) )
        self.debugDraw.surface = self.screen
        self.doDebugDraw = False
    
        self.screenSize = Box2D.b2Vec2(*self.screen.get_size())
        
        try:
            self.fontFPS = pygame.font.Font(None, 15)
        except IOError:
            try:
                self.fontFPS = pygame.font.Font("freesansbold.ttf", 15)
            except IOError:
                print "Unable to load default font or 'freesansbold.ttf'"
        try:
            self.fontDefault = pygame.font.Font(None, 30)
        except IOError:
            try:
                self.fontDefault = pygame.font.Font("freesansbold.ttf", 30)
            except IOError:
                print "Unable to load default font or 'freesansbold.ttf'"
        try:
            verdana = pygame.font.match_font('Verdana', italic = True)
            self.fontTransition = pygame.font.Font(verdana, 50)
        except IOError:
            try:
                self.fontTransition = pygame.font.Font("freesansbold.ttf", 50)
            except IOError:
                print "Unable to load default font or 'freesansbold.ttf'"
      
        self.creditsRenderer = CreditsRenderer.CreditsRenderer(self.screen)
        
        level1 = LevelDefinition()
        level1.title = "Welcome to Idaion"
        level1.dataFileName = path.join(self.artdir, 'level1.svg')
        level1.startPoint = (60, -80)
        level1.initialCameraPoint = (50, -50)
        self.levels.append(level1)
        
        level2 = LevelDefinition()
        level2.title = "Two birds, one stone"
        level2.dataFileName = path.join(self.artdir, 'twobirdsonestone2.svg')
        level2.startPoint = (60, -150)
        level2.initialCameraPoint = (80, -50)
        self.levels.append(level2)
        
        level4 = LevelDefinition()
        level4.title = "Ring of Fire"
        level4.dataFileName = path.join(self.artdir, 'rocketarse.svg')
        level4.startPoint = (70, -200) 
        level4.initialCameraPoint = (60, -80)
        self.levels.append(level4)
        
        level5 = LevelDefinition()
        level5.title = "Round the U-Bend"
        level5.dataFileName = path.join(self.artdir, 'ubend.svg')
        level5.startPoint = (45, -65) 
        level5.initialCameraPoint = (70, -80)
        self.levels.append(level5)
        
        level1.nextLevel = level2
        level2.nextLevel = level4        
        level4.nextLevel = level5
        level5.nextLevel = None
        
        # Load a title image
        self.titleScreenImage = pygame.image.load(path.join(self.artdir, 'icarus_title_screen.jpg')).convert()
        self.endScreenImage = pygame.image.load(path.join(self.artdir, 'endscreen.jpg')).convert()
        
        # Load images and animations
        self.loadBirdFrames()       
        self.loadFeathers()
        self.loadIcarus()
        
        # Load sounds
        self.loadSounds()
    
    def checkAndFixGarysDodgyPolygons(self, poly):
        accum = 0.0
        for i in range(len(poly)):
            j = (i + 1) % len(poly)
            accum += poly[j][0] * poly[i][1] - poly[i][0] * poly[j][1]
        if accum > 0:
            poly.reverse()    
        
    def loadBirdFrames(self):
        birddir = path.abspath(path.join(self.artdir, 'bird'))
        birdimages = [img for img in os.listdir(birddir) if ".png" in img]
        birdimages.sort()       
        
        reduction = 6
        for png in birdimages:
            birdimg = pygame.image.load(path.join(birddir, png)).convert_alpha()
            scaledbird = pygame.transform.smoothscale(
                            birdimg, (birdimg.get_width()/reduction, birdimg.get_height()/reduction))
            self.birdframes.append(scaledbird)     
            
    def loadFeathers(self):
        featherimages = [path.join(self.artdir, f) for f in os.listdir(self.artdir) if f.startswith("feather")]
        for img in featherimages:
            featherimage = pygame.image.load(path.abspath(img)).convert_alpha()
            self.feathers.append(featherimage)     
            
    def loadIcarus(self):
        icarusdir = path.join(self.artdir, 'icarus')
        icarusimages = [path.join(icarusdir, i) 
                        for i in os.listdir(icarusdir) 
                        if i.startswith("frame")]
        for img in icarusimages:
            icarusimage = pygame.image.load(img).convert_alpha()
            scaledicarus = pygame.transform.smoothscale(
                            icarusimage, 
                            (icarusimage.get_width()/8, icarusimage.get_height()/8))
            self.icarusframesRight.append(scaledicarus)
            flippedicarus = pygame.transform.flip(scaledicarus, True, False)
            self.icarusframesLeft.append(flippedicarus)
            
    def loadSounds(self):
        for file in os.listdir(self.sounddir):
            if file.endswith(".ogg"):
                self.soundbank[file[:-4].lower()] = pygame.mixer.Sound(path.join(self.sounddir, file))
    
    playFanfare = False
    def setNextLevel(self, nextLevel):
        if self.playFanfare:
            self.soundbank['stagecomplete'].play()
        self.playFanfare = True # don't play fanfare for the first level

        if not nextLevel:
            # go to the WIN screen
            self.currentGameState = eGameState.stateYouWin
            
            self.playFanfare = False
        else:
            # go to next level
            self.nextLevel = nextLevel
            self.currentGameState = eGameState.stateLevelTransition
    
    def touchedFire(self, icarus):
        self.spawnFeathers(self.icarus.numFeathers, self.icarus.body.GetPosition())
        self.icarus.numFeathers = 0
        
        if icarus.burningTimer == 0:
            self.soundbank['setonfire'].play()
            icarus.burningTimer = 200
            
        
    def loadNextLevel(self):
        
        self.currentLevel = self.nextLevel
        self.nextLevel = None
        
        parser = svgparser.SVGParser(self.currentLevel.dataFileName, 0.1)
        
        # Clear out all actors
        for actor in self.actors:
            actor.Kill()
        self.actors = []
        
        # Reset the camera
        self.viewCenter = [point * 10 for point in self.currentLevel.initialCameraPoint]
        
        # Load walls
        cavepolys = parser.pointsFromLayer('cave', True)
        self.addWallActorsFromPoints(cavepolys)

        # Create icarus
        self.icarus = IcarusActor.IcarusActor(
                self.world, 
                self.debugDraw, 
                self.currentLevel.startPoint,
                self.onCollectedFeather, 
                self.icarusframesRight, 
                self.icarusframesLeft,
                self.soundbank)
        self.actors.append(self.icarus)        

        # load background image
        self.backgroundimage = pygame.image.load(parser.imageFromLayer('background')).convert()
        
        # load birds
        self.birdpoints = parser.singlePointsFromLayer('birds')
        for point in self.birdpoints:
            self.actors.append(BirdActor.BirdActor(self.world, self.debugDraw, point, self.birdframes, self.onShotBird))
            
        #load wind
        windlines = parser.pointsFromLayer('wind', False)        
        widths = parser.attributeFromLayerStyle('wind', "stroke-width")        
        strengths = parser.attributeFromLayerStyle('wind', "stroke-opacity")
        print "windlines %s" % windlines
        if windlines != None and widths != None and strengths != None:
            winds = zip(windlines, widths, strengths)
            for line, width, strength in winds:
                print "windline %s %d %d" % (line, width, strength)
                self.actors.append(WindActor.WindActor(line, width/10, strength * 200, self.world, self.debugDraw, self.actors))

        triggerpolys = parser.pointsFromLayer('endtrigger', True)
        if triggerpolys != None:
            for poly in triggerpolys:
                self.checkAndFixGarysDodgyPolygons(poly)
                actor = TriggerActor.TriggerActor((0,0), poly, self.world, self.debugDraw, self.setNextLevel, self.currentLevel.nextLevel)
                self.actors.append(actor)

        firepolys = parser.pointsFromLayer('fire', True)
        if firepolys != None:
            for poly in firepolys:
                self.checkAndFixGarysDodgyPolygons(poly)
                actor = TriggerActor.TriggerActor((0,0), poly, self.world, self.debugDraw, self.touchedFire, self.icarus)
                self.actors.append(actor)

        thermalpolys = parser.pointsFromLayer('thermals', True)
        if thermalpolys != None:
            for poly in thermalpolys:
                self.checkAndFixGarysDodgyPolygons(poly)
                actor = ThermalActor.ThermalActor((0,0), poly, self.world, self.debugDraw, self.actors)
                self.actors.append(actor)
        
        # play ambient noise
        self.soundbank['cave'].stop()
        self.soundbank['cave'].play()

    def addWallActorsFromPoints(self, polylist):
        for poly in polylist:
            self.checkAndFixGarysDodgyPolygons(poly)
            actor = WallActor.WallActor((0,0), poly, self.world, self.debugDraw)
            self.actors.append(actor)
        
    def setCenter(self, value):
        """
        Updates the view offset based on the center of the screen.
        
        Tells the debug draw to update its values also.
        """
        if isinstance(value, Box2D.b2Vec2):
            self._viewCenter = value.copy()
        elif isinstance(value, (list, tuple)):
            self._viewCenter = Box2D.b2Vec2( *value )
        else:
            raise ValueError, 'Expected b2Vec2 or sequence'

        self._viewOffset = self._viewCenter - self.screenSize/2

        self.debugDraw._setValues(self.viewZoom, self.viewCenter, self.viewOffset, self.screenSize.x, self.screenSize.y)
    
    def setZoom(self, zoom):
        self._viewZoom = zoom
        self.debugDraw.viewZoom = zoom

    viewZoom   = property(lambda self: self._viewZoom, setZoom,
                           doc='Zoom factor for the display')
    viewCenter = property(lambda self: self._viewCenter, setCenter, 
                           doc='Screen center in camera coordinates')
    viewOffset = property(lambda self: self._viewOffset,
                           doc='The offset of the top-left corner of the screen')

    def handleContact(self, contact):
        actor1 = contact.actor1
        actor2 = contact.actor2
        
        if actor1.isAlive and actor2.isAlive:
            # Handle contacts where one party is icarus 
            if isinstance(actor1, IcarusActor.IcarusActor):
                actor1.HandleContact(contact)
            elif isinstance(actor2, IcarusActor.IcarusActor):
                actor2.HandleContact(contact.GetSwapped())
            
            # Handle contacts where one party is a bird
            # (Possible that both parties are birds)
            if isinstance(actor1, BirdActor.BirdActor):
                actor1.HandleContact(contact)
            if isinstance(actor2, BirdActor.BirdActor):
                actor2.HandleContact(contact.GetSwapped())
    
            # Handle contacts where one party is a feather 
            if isinstance(actor1, FeatherActor.FeatherActor):
                actor1.HandleContact(contact)
            elif isinstance(actor2, FeatherActor.FeatherActor):
                actor2.HandleContact(contact.GetSwapped())
    
            # Handle contacts where one party is a bullet
            if isinstance(actor1, BulletActor.BulletActor):
                actor1.HandleContact(contact)
            elif isinstance(actor2, BulletActor.BulletActor):
                actor2.HandleContact(contact.GetSwapped())
    
            # Handle contacts where one party is a particle
            if isinstance(actor1, ParticleActor.ParticleActor):
                actor1.HandleContact(contact)
            elif isinstance(actor2, ParticleActor.ParticleActor):
                actor2.HandleContact(contact.GetSwapped())

    def run(self):
        """
        Main loop.

        Continues to run while checkEvents indicates the user has 
        requested to quit.

        Updates the screen and tells the GUI to paint itself.
        """
        running = True
        clock = pygame.time.Clock()

        while running:
            
            pygame.event.pump()
            running = self.checkEvents()
            
            self.screen.fill( (0,0,0) )     
            
            # Check (poll) for pressed keys            
            keys = pygame.key.get_pressed()
            
            if self.currentGameState == eGameState.stateTitle:
                self.screen.blit(self.titleScreenImage, (0, 0))
                self.DrawString(720, 540, self.fontDefault, "Press any key to start...", (0,0,0))
                
            elif self.currentGameState == eGameState.stateLevelTransition:
                if self.nextLevel != None:
                    self.loadNextLevel()
                self.DrawString(100, 100, self.fontTransition, self.currentLevel.title)
                self.DrawString(720, 540, self.fontDefault, "Press any key to continue...")
                
            elif self.currentGameState == eGameState.statePlayingLevel:
                # Run the physics update step 
                self.Step()

                # move the camera, if required, so icarus is always in the middle third
                icarusScreenPos = self.debugDraw.toScreen_v(self.icarus.body.GetPosition())
                if icarusScreenPos[0] < 400:
                    # move camera left
                    self.viewCenter -= (400 - icarusScreenPos[0], 0)
                elif icarusScreenPos[0] > 600:
                    # move camera right
                    self.viewCenter += (icarusScreenPos[0] - 600, 0)
                if icarusScreenPos[1] < 200:
                    # move camera down
                    self.viewCenter += (0, 200 - icarusScreenPos[1])
                elif icarusScreenPos[1] > 300:
                    # move camera up
                    self.viewCenter -= (0, icarusScreenPos[1] - 300)
                                
                self.screen.blit(self.backgroundimage, (-self.viewOffset.x, self.viewOffset.y + self.screenSize.y))

                # remove any dead actors            
                for actor in self.actors:
                    if actor.isAlive == False:
                        self.actors.remove(actor)
    
                for actor in self.actors:
                    if actor.isAlive:
                        actor.Update(keys)
                    
                for contact in self.contactPoints:
                    if contact.actor1.isAlive and contact.actor2.isAlive:
                        self.handleContact(contact)  
    
                for actor in self.actors:
                    if actor.isAlive:
                        actor.Render(self.doDebugDraw)
                    
                self.DrawString(10, 10, self.fontDefault, self.currentLevel.title)
                self.DrawString(10, 30, self.fontDefault, "Feathers: %d" % self.icarus.numFeathers)
                
                # Show the FPS
                #self.DrawString(900, 10, self.fontFPS, "FPS %d" % self.fps)
    
            elif self.currentGameState == eGameState.stateYouWin:
                self.screen.blit(self.endScreenImage, (0,0))                
                self.creditsRenderer.Render()
                                            
            pygame.display.flip()
            clock.tick(self.updateRate)
            self.fps = clock.get_fps()

    def checkEvents(self):
        """
        Check for pygame events (mainly keyboard/mouse events).
        """
        for event in pygame.event.get():
            if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE):
                return False
            elif event.type == KEYDOWN:
                self.OnKeyDownEvent(event.key)
            elif event.type == KEYUP:
                self.OnKeyUpEvent(event.key)
            elif event.type == MOUSEBUTTONDOWN:
                if event.button == 1: #left
                    self.fireBullet()
                if event.button == 3: #right
                    self.rMouseDown = True
                #elif event.button ==4: # zoom on mousewheel
                #    self.viewZoom *= 1.1
                #elif event.button == 5:
                #    self.viewZoom /= 1.1
#            elif event.type == MOUSEBUTTONUP:
#                if event.button == 3: #right
#                    self.rMouseDown = False
#            elif event.type == MOUSEMOTION:
#                if self.rMouseDown:
#                    self.viewCenter -= (event.rel[0], -event.rel[1])
        return True
    
    def fireBullet(self):
        if self.currentGameState != eGameState.statePlayingLevel:
            return
        self.soundbank['slingshot_really_short'].play()
        mousepos = self.ConvertScreenToWorld(*pygame.mouse.get_pos())        
        if mousepos is None: # handle off-screen click
            return
        print mousepos
        icaruspos = self.icarus.body.GetPosition()
        velocity = mousepos - icaruspos # normalise, then add constant speed?
        offset = copy(velocity)
        offset.Normalize()
        offset *= 1.8
        bullet = BulletActor.BulletActor(
            self.world, self.debugDraw, icaruspos + offset, velocity)
        self.actors.append(bullet)    
    
    def onShotBird(self, deadbird):
        self.soundbank['birdexplosion_short'].play()
        self.spawnFeathers(4, deadbird.body.position)
    
    def spawnFeathers(self, numfeathers, position):
        for i in range(numfeathers):
            featherimage = random.randint(0, len(self.feathers)-1)
            self.actors.append(FeatherActor.FeatherActor(
                                 self.world, 
                                 self.debugDraw, 
                                 position, 
                                 self.feathers[featherimage]))        
    
    def onCollectedFeather(self, feather):
        self.soundbank['feather_pickup'].play()
        pass
    
    def ConvertScreenToWorld(self, x, y):
        """ Return a b2Vec2 in world coordinates of the passed in screen coordinates x, y """
        return Box2D.b2Vec2((x + self.viewOffset.x) / self.viewZoom, 
                           ((self.screenSize.y - y + self.viewOffset.y) / self.viewZoom))
        
    def OnKeyDownEvent(self, key):
#        if key == K_p:
#            self.doDebugDraw = not self.doDebugDraw
            
        if self.currentGameState == eGameState.stateTitle:
            # A key was pressed at the title screen, so begin the game
            self.setNextLevel(self.levels[0])
        elif self.currentGameState == eGameState.stateLevelTransition:
            # A key was pressed at the transition screen, so begin the level
            self.currentGameState = eGameState.statePlayingLevel
#        elif self.currentGameState == eGameState.statePlayingLevel:
#            # Level hack!!!
#            if key == K_1:
#                self.setNextLevel(self.levels[0])
#            elif key == K_2:
#                self.setNextLevel(self.levels[1])
#            elif key == K_3:
#                self.setNextLevel(self.levels[2])
#            elif key == K_4:
#                self.setNextLevel(self.levels[3])
#            elif key == K_5:
#                self.setNextLevel(self.levels[4])
#            elif key == K_r:
#                self.setNextLevel(self.currentLevel)
#            elif key == K_c:
#                self.currentGameState = eGameState.stateYouWin
        elif self.currentGameState == eGameState.stateYouWin:
            # A key was pressed at the you-win screen, so go back to title screen
            self.currentGameState = eGameState.stateTitle
            
        return

    def OnKeyUpEvent(self, key):
        # Do nothing (for now)
        return

    def Step(self):
        """
        The main physics step.

        Takes care of physics drawing (callbacks are executed after the world.Step() )
        and drawing additional information.
        """

        timeStep = 1.0 / self.updateRate
        

        # Set the flags based on what the settings show (uses a bitwise or mask)
        flag_info = [
            (True, Box2D.b2DebugDraw.e_shapeBit),
            (True, Box2D.b2DebugDraw.e_jointBit),
            (True, Box2D.b2DebugDraw.e_controllerBit),
            (False, Box2D.b2DebugDraw.e_coreShapeBit),
            (False, Box2D.b2DebugDraw.e_aabbBit),
            (False, Box2D.b2DebugDraw.e_obbBit),
            (False, Box2D.b2DebugDraw.e_pairBit),
            (False, Box2D.b2DebugDraw.e_centerOfMassBit),
            ]

        flags = 0
        for setting, flag in flag_info:
            if setting:
                flags |= flag

        self.debugDraw.SetFlags(flags)

        # Set the other settings that aren't contained in the flags
        self.world.SetWarmStarting(True)
        self.world.SetContinuousPhysics(True)

        # Reset the collision points
        self.contactPoints = []

        # Tell Box2D to step
        velocityIterations = 10
        positionIterations = 8
        self.world.Step(timeStep, velocityIterations, positionIterations)
        self.world.Validate()

        # Destroy bodies that have left the world AABB (can be removed if not using pickling)
        for obj in self.destroyList:
            self.world.DestroyBody(obj)
        self.destroyList = []
        
    def DrawString(self, x, y, font, str, color=(229,183,153,255)):
        """
        Draw some text, str, at screen coordinates (x, y).
        """
        if not font: return 
        text = font.render(str, True, color)
        self.screen.blit(text, (x,y))

def main(game_class):
    """
    Loads the game class and executes it.
    """
    print "Loading..."
    game = game_class()
    game.run()

if __name__=="__main__":
    main(Game)
