import os
from xml.etree.ElementTree import ElementTree

class SVGParser:
    
    ''' Parses SVG files to get lists of points in paths '''
    
    # some common namespaces in inkscape svg files
    inkns = '{http://www.inkscape.org/namespaces/inkscape}'
    w3ns = '{http://www.w3.org/2000/svg}'
    sodins = '{http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd}'
    
    def __init__(self, svgpath, scaling=1):        
        self.svgpath = svgpath
        self.svgdir = os.path.dirname(svgpath)
        self.eltree = ElementTree()
        self.eltree.parse(svgpath)
        self.scaling = scaling        
                
    def getLayer(self, layerName):
        # get the 'g' tag with 'inkscape:label="[layer]"'
        allgs = self.eltree.findall(self.w3ns + 'g')        
        for g in allgs:
            if g.attrib[self.inkns + 'label'] == layerName:
                return g # returns first match - keep layer names unique!
    
    def pointsFromLayer(self, layerName, discardLast):
        ''' Returns a list of points for paths in a given layer '''
        glayer = self.getLayer(layerName)
        if not glayer:
            # no layer entries found
            return
        # parse the points from the 'd' attrb of the 'path' child tag
        paths = glayer.findall(self.w3ns + 'path')
        polylist = []
        for path in paths:
            dattr = path.attrib['d']
            polylist.append(self.parsePoints(dattr, discardLast))
        return polylist
    
    def singlePointsFromLayer(self, layerName):
        ''' Gets the first point of every path - useful for things like birds '''
        polys = self.pointsFromLayer(layerName, False)
        return [p[0] for p in polys]
    
    def parsePoints(self, pointList, discardLast):
        ''' Takes a list of points from an SVG file, 
        returns a list of point tuples in draw order '''        
        pttrim = pointList[2:-2] # remove the leading M and trailing Z
        ptspl = pttrim.split("L")
        strpts = [pt.split(",") for pt in ptspl]
        intpts = []
        if discardLast:
            # discard last point - Box2D doesn't work with closed paths
            for pt in strpts[:-1]: 
                # invert the y - Box2D uses bottom-left (0,0), inkscape uses top-left
                intpts.append((float(pt[0])*self.scaling, -float(pt[1])*self.scaling))
        else:
            for pt in strpts: 
                # invert the y - Box2D uses bottom-left (0,0), inkscape uses top-left
                intpts.append((float(pt[0])*self.scaling, -float(pt[1])*self.scaling))
        return intpts
    
    def attributeFromLayerStyle(self, layerName, attributeName):
        layer = self.getLayer(layerName)
        if layer == None: return None
        paths = layer.findall(self.w3ns + 'path')
        widths = []
        for path in paths:
            styleAttrib = path.attrib['style']
            styles = styleAttrib.split(";")            
            for style in styles:
                if attributeName in style:
                    widths.append(float(style.split(":")[1]))
        return list(widths)
    
    def imageFromLayer(self, layerName):
        ''' Gets an image (e.g. the level backgroud) from a given layer '''
        layer = self.getLayer(layerName)
        image = layer.find(self.w3ns + 'image')
        return os.path.join(self.svgdir, image.attrib['{http://www.w3.org/1999/xlink}href'])
    
    